#!/usr/bin/env python

# Author: Arman Gungor
# Name: Word_Authors.py
# Version: 0.1
# Date: 11/6/2014
#
# Copyright (c) 2014 Arman Gungor. All rights reserved.

import struct
import re
import olefile
import sys

def parse_revision_log(revision_log):
    revisions = []
    try:
        string_count =  struct.unpack('<H',revision_log[2:4])[0]
        current_location = 6

        for i in range(1, string_count/2+1):
            strlen = struct.unpack('<H',revision_log[current_location:current_location+2])[0]
            current_location += 2
            auth = re.sub('\x00', '', unicode(revision_log[current_location:current_location+strlen*2], 'UTF16'))
            current_location += strlen*2
            strlen = struct.unpack('<H',revision_log[current_location:current_location+2])[0]
            current_location += 2
            path = re.sub('\x00', '', unicode(revision_log[current_location:current_location+strlen*2], 'UTF16'))
            current_location += strlen*2
            revisions.append((auth, path))
    except:
        print "Cannot parse revision log"
    return revisions
    
def process_document(argv):
    file_name = ''
    metadata_dump = False

    if len(argv) == 0 or len(argv) > 2 or (len(argv) == 2 and argv[1] != '-m'):
        print """
This script parses Word documents and displays last 10 author information.

Usage: Word_Authors.py <inputfile> [-m]

Options:
-m : Dump metadata
"""
        sys.exit()
    else:
        file_name = argv[0]
        if len(argv) == 2:
            metadata_dump = True

    try:
        with open(file_name, 'rb') as target_file:
            signature = target_file.read(8)
            
            # Check document signature to make sure it is a valid OLE document
            if signature != '\xD0\xCF\x11\xE0\xA1\xB1\x1A\xE1':
                print "Invalid document type"
            else:
                # FIB starts at 0x0200
                # 0x000A offset from FIB contains fWhichTblStm, which indicates if the 0Table or the 1Table stream is used
                target_file.seek(0x020A)
                which_table = struct.unpack('<H',target_file.read(2))[0]

                if which_table & 0x0200:
                    stream_name = '1Table'
                else:
                    stream_name = '0Table'

                # 0x02D2 offset from FIB contains fcSttbSavedBy, which is the 4-byte offset indicating revision log location
                # 0x02D6 offset from FIB contains lcbSttbSavedBy, which is a 4-byte value indicating revision log size

                target_file.seek(0x04D2)
                saved_by_offset = struct.unpack('<l',target_file.read(4))[0]
                saved_by_length = struct.unpack('<l',target_file.read(4))[0]

                # Read revision log from Table Stream
                ole = olefile.OleFileIO(file_name)
                table_stream = ole.openstream(stream_name)
                table_stream.seek(saved_by_offset)
                revision_log = table_stream.read(saved_by_length)

                # Parse revision log
                parsed_revisions = parse_revision_log(revision_log)
                print "---- Last Author Information ----"
                i = 1
                for record in parsed_revisions:
                    print i,"- Author: ", record[0], "Path: ", record[1]
                    i += 1
                if metadata_dump:
                    print    
                    print "---- Other Metadata ----"
                    ole.get_metadata().dump()
                ole.close()
    except IOError:
        print "Cannot open file: ", file_name

if __name__ == '__main__':
    process_document(sys.argv[1:])
